from cpl_core.configuration.configuration_model_abc import ConfigurationModelABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum


class FeatureFlagsSettings(ConfigurationModelABC):
    _flags = {
        # modules
        FeatureFlagsEnum.achievements_module.value: False,  # 14.06.2023 #268
        FeatureFlagsEnum.api_module.value: False,  # 13.10.2022 #70
        FeatureFlagsEnum.admin_module.value: False,  # 02.10.2022 #48
        FeatureFlagsEnum.auto_role_module.value: True,  # 03.10.2022 #54
        FeatureFlagsEnum.base_module.value: True,  # 02.10.2022 #48
        FeatureFlagsEnum.boot_log_module.value: True,  # 02.10.2022 #48
        FeatureFlagsEnum.core_module.value: True,  # 03.10.2022 #56
        FeatureFlagsEnum.core_extension_module.value: True,  # 03.10.2022 #56
        FeatureFlagsEnum.data_module.value: True,  # 03.10.2022 #56
        FeatureFlagsEnum.database_module.value: True,  # 02.10.2022 #48
        FeatureFlagsEnum.moderator_module.value: False,  # 02.10.2022 #48
        FeatureFlagsEnum.permission_module.value: True,  # 02.10.2022 #48
        FeatureFlagsEnum.config_module.value: True,  # 19.07.2023 #127
        # features
        FeatureFlagsEnum.api_only.value: False,  # 13.10.2022 #70
        FeatureFlagsEnum.presence.value: True,  # 03.10.2022 #56
        FeatureFlagsEnum.version_in_presence.value: False,  # 21.03.2023 #253
    }

    def __init__(self, **kwargs: dict):
        ConfigurationModelABC.__init__(self)

        if len(kwargs.keys()) == 0:
            return

        for flag in [f.value for f in FeatureFlagsEnum]:
            self._load_flag(kwargs, FeatureFlagsEnum(flag))

    def get_flag(self, key: FeatureFlagsEnum) -> bool:
        if key.value not in self._flags:
            return False
        return self._flags[key.value]

    def _load_flag(self, settings: dict, key: FeatureFlagsEnum):
        if key.value not in settings:
            return

        self._flags[key.value] = bool(settings[key.value])
