CREATE TABLE IF NOT EXISTS `CFG_TechnicianHistory`
(
    `Id`    BIGINT(20)  NOT NULL,
    `HelpCommandReferenceUrl` VARCHAR(255) NOT NULL,
    `WaitForRestart` BIGINT NOT NULL DEFAULT 8,
    `WaitForShutdown` BIGINT NOT NULL DEFAULT 8,
    `CacheMaxMessages` BIGINT NOT NULL DEFAULT 1000000,
    `Deleted` BOOL DEFAULT FALSE,
    `DateFrom` DATETIME(6) NOT NULL,
    `DateTo` DATETIME(6) NOT NULL
);

DROP TRIGGER IF EXISTS `TR_CFG_TechnicianUpdate`;

CREATE TRIGGER `TR_CFG_TechnicianUpdate`
    AFTER UPDATE
    ON `CFG_Technician`
    FOR EACH ROW
BEGIN
    INSERT INTO `CFG_TechnicianHistory` (
        `Id`,
        `HelpCommandReferenceUrl`,
        `WaitForRestart`,
        `WaitForShutdown`,
        `CacheMaxMessages`,
        `DateFrom`,
        `DateTo`
    )
    VALUES (
        OLD.Id,
        OLD.HelpCommandReferenceUrl,
        OLD.WaitForRestart,
        OLD.WaitForShutdown,
        OLD.CacheMaxMessages,
        OLD.LastModifiedAt,
        CURRENT_TIMESTAMP(6)
    );
END;

DROP TRIGGER IF EXISTS `TR_CFG_TechnicianDelete`;

CREATE TRIGGER `TR_CFG_TechnicianDelete`
    AFTER DELETE
    ON `CFG_Technician`
    FOR EACH ROW
BEGIN
    INSERT INTO `CFG_TechnicianHistory` (
        `Id`,
        `HelpCommandReferenceUrl`,
        `WaitForRestart`,
        `WaitForShutdown`,
        `CacheMaxMessages`,
        `Deleted`,
        `DateFrom`,
        `DateTo`
    )
    VALUES (
        OLD.Id,
        OLD.HelpCommandReferenceUrl,
        OLD.WaitForRestart,
        OLD.WaitForShutdown,
        OLD.CacheMaxMessages,
        TRUE,
        OLD.LastModifiedAt,
        CURRENT_TIMESTAMP(6)
    );
END;