import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnVoiceStateUpdateABC

from bot_core.helper.event_checks import EventChecks
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server_config import ServerConfig
from modules.base.service.event_service import EventService


class BaseOnVoiceStateUpdateEventScheduledEventBonus(OnVoiceStateUpdateABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        events: EventService,
        db: DatabaseContextABC,
    ):
        OnVoiceStateUpdateABC.__init__(self)
        self._config = config
        self._logger = logger
        self._servers = servers
        self._users = users
        self._events = events
        self._db = db

        self._logger.info(__name__, f"Module {type(self)} loaded")

    @EventChecks.check_is_ready()
    async def on_voice_state_update(
        self,
        member: discord.Member,
        before: discord.VoiceState,
        after: discord.VoiceState,
    ):
        self._logger.debug(__name__, f"Module {type(self)} started")
        if member.bot or after.channel is None:
            self._logger.debug(__name__, f"Module {type(self)} stopped")
            return

        active_event = self._events.get_active_event_by_channel_id(after.channel.id)
        if active_event is None:
            self._logger.debug(__name__, f"Module {type(self)} stopped")
            return

        server = self._servers.get_server_by_discord_id(member.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)
        if active_event.participants.any(lambda x: x.id == user.id):
            self._logger.debug(__name__, f"Module {type(self)} stopped")
            return

        settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{server.discord_id}")
        user.xp += settings.xp_per_event_participation
        self._users.update_user(user)
        self._db.save_changes()
        active_event.participants.append(user)

        self._logger.debug(__name__, f"Module {type(self)} stopped")
