import { Component, OnInit } from "@angular/core";
import { FormBuilder } from "@angular/forms";
import { TranslateService } from "@ngx-translate/core";
import { catchError } from "rxjs/operators";
import { GuiService } from "src/app/services/gui/gui.service";
import { SettingsService } from "src/app/services/settings/settings.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { ToastService } from "src/app/services/toast/toast.service";
import { throwError } from "rxjs";
import { Query, ServerConfigQuery } from "../../../../../../models/graphql/query.model";
import { Queries } from "../../../../../../models/graphql/queries.model";
import { DataService } from "../../../../../../services/data/data.service";
import { ServerConfigMutationResult } from "../../../../../../models/graphql/result.model";
import { Mutations } from "../../../../../../models/graphql/mutations.model";
import { AuthService } from "../../../../../../services/auth/auth.service";
import { ServerConfig } from "../../../../../../models/config/server-config.model";
import { Server } from "../../../../../../models/data/server.model";
import { ActivatedRoute } from "@angular/router";
import { Table } from "primeng/table";

type AFKChannelId = {
  id: number;
  value: string;
};

@Component({
  selector: "app-config",
  templateUrl: "./config.component.html",
  styleUrls: ["./config.component.scss"]
})
export class ConfigComponent implements OnInit {
  config: ServerConfig = {
    messageDeleteTimer: 0,
    afkChannelIds: [],
    moderatorRoleIds: [],
    adminRoleIds: []
  };


  afkChannelIds: AFKChannelId[] = [];
  clonedAfkChannelIds: { [s: number]: AFKChannelId } = {};

  private server: Server = {};

  constructor(
    private data: DataService,
    private settingsService: SettingsService,
    private spinnerService: SpinnerService,
    private guiService: GuiService,
    private formBuilder: FormBuilder,
    private toastService: ToastService,
    private translate: TranslateService,
    private authService: AuthService,
    private spinner: SpinnerService,
    private route: ActivatedRoute
  ) {
  }

  ngOnInit(): void {
    this.spinnerService.showSpinner();

    this.data.getServerFromRoute(this.route).then(async server => {
      this.server = server;
      this.loadConfig();
    });
  }

  loadConfig() {
    this.data.query<ServerConfigQuery>(Queries.serverConfigQuery, {
        serverId: this.server.id
      },
      (data: Query) => {
        return data.servers[0];
      }).subscribe(data => {
      this.config = data.config;

      let id = 0;
      for (const afkChannelId of this.config.afkChannelIds ?? []) {
        this.afkChannelIds.push({ id: id, value: afkChannelId });
        id++;
      }

      this.spinnerService.hideSpinner();
    });
  }

  saveServerConfig() {
    this.spinner.showSpinner();
    this.data.mutation<ServerConfigMutationResult>(Mutations.updateServerConfig, {
        id: this.config.id,
        messageDeleteTimer: this.config.messageDeleteTimer,
        notificationChatId: this.config.notificationChatId,
        maxVoiceStateHours: this.config.maxVoiceStateHours,
        xpPerMessage: this.config.xpPerMessage,
        xpPerReaction: this.config.xpPerReaction,
        maxMessageXpPerHour: this.config.maxMessageXpPerHour,
        xpPerOntimeHour: this.config.xpPerOntimeHour,
        xpPerEventParticipation: this.config.xpPerEventParticipation,
        xpPerAchievement: this.config.xpPerAchievement,
        afkCommandChannelId: this.config.afkCommandChannelId,
        helpVoiceChannelId: this.config.helpVoiceChannelId,
        teamChannelId: this.config.teamChannelId,
        loginMessageChannelId: this.config.loginMessageChannelId,
        afkChannelIds: this.config.afkChannelIds,
        moderatorRoleIds: this.config.moderatorRoleIds,
        adminRoleIds: this.config.adminRoleIds
      }
    ).pipe(catchError(err => {
      this.spinner.hideSpinner();
      return throwError(err);
    })).subscribe(result => {
      this.spinner.hideSpinner();
      this.toastService.success(this.translate.instant("view.server.config.message.technician_config_create"), this.translate.instant("view.server.config.message.technician_config_create_d"));
    });
  }
}
